@@ -42,6 +42,7 @@ import os
 import webbrowser
 import ConfigParser
 import pycurl
+import pango
 
 
 #Initially I found I'd hang the whole interface if I was having network probs
@@ -62,7 +63,7 @@ class Witter():
         self.program = hildon.Program()
         self.program.__init__()
 
-        self.textcolour="#000000"
+
         # set name of application: this shows in titlebar
         gtk.set_application_name("Witter")
         #Set the Glade file
@@ -96,7 +97,7 @@ class Witter():
         self.namecolour = self.textcolour
         self.tweetcolour = self.textcolour
         
-        self.defaultwidth = 500
+        self.defaultwidth = 790
         #default to colours above, but check if we're on fremantle and change
         #to appropriate colours if we are
         self.checkVersion()
@@ -155,35 +156,38 @@ class Witter():
         self.treeview.set_model(self.liststore)
         # create the TreeViewColumn to display the data, I decided on two colums
         # one for name and the other for the tweet
-        self.tvcname = gtk.TreeViewColumn('Name')
-        self.tvctweet = gtk.TreeViewColumn('Tweet')
+        #self.tvcname = gtk.TreeViewColumn('Name')
+        cell = gtk.CellRendererText()
+        self.tvctweet = gtk.TreeViewColumn('Pango Markup', cell, markup=2)
+
+        #self.tvctweet = gtk.TreeViewColumn('Tweet')
         # add the two tree view columns to the treeview
-        self.treeview.append_column(self.tvcname)
+        #self.treeview.append_column(self.tvcname)
         self.treeview.append_column(self.tvctweet)
         # we need a CellRendererText to render the data
-        self.cell = gtk.CellRendererText()
+        
         # add the cell renderer to the columns
-        self.tvcname.pack_start(self.cell, True)
-        self.tvctweet.pack_start(self.cell,True)
+        #self.tvcname.pack_start(cell, True)
+        #self.tvctweet.pack_start(cell,True)
         # set the cell "text" attribute to column 0 - retrieve text
         # from that column in liststore and treat it as the text to render
         # in this case it's the name of a tweeter
-        self.tvcname.add_attribute(self.cell, 'text', 0)
+        #self.tvcname.add_attribute(self.cell, 'text', 0)
         # we then use the second field of our liststore to hold the colour for
         # the 'name' text
-        self.tvcname.add_attribute(self.cell, 'foreground', 1)
+        #self.tvcname.add_attribute(self.cell, 'foreground', 1)
         # next we add a mapping to the tweet column, again the third field
         # in our list store is the tweet text
-        self.tvctweet.add_attribute(self.cell, 'text',2)
+        self.tvctweet.add_attribute(cell, 'text',2)
         # and the fourth is the colour of the tweet text 
-        self.tvctweet.add_attribute(self.cell, 'foreground', 3)
+        #self.tvctweet.add_attribute(cell, 'foreground', 3)
         # we start up non-fullscreen, and we want the tweets to appear without
         # scrolling left-right (well I wanted that) so I set a wrap width for
         # the text being rendered
-        self.cell.set_property('wrap-width', self.defaultwidth)
+        cell.set_property('wrap-width', self.defaultwidth)
         # make it searchable (I found this in an example and thought I might use it
         # but currently I make no use of this setting
-        self.treeview.set_search_column(0)
+        self.treeview.set_search_column(2)
         self.treeview.set_rules_hint(True)
        
         self.treeview.set_property('enable-grid-lines',True)
@@ -209,8 +213,7 @@ class Witter():
         # self.treeview.connect("changed", self.build_menu, None);
         self.treeview.tap_and_hold_setup(self.urlmenu, callback=gtk.tap_and_hold_menu_position_top)
        
-        #we are going to keep a list of the URLs we've found and used in the menu
-        self.ListedURLS=""
+        
         
         
     def quit(self, *args):
@@ -366,19 +369,33 @@ class Witter():
         # ...and install it globally so it can be used with urlopen.
         urllib2.install_opener(opener)
         #switch on whether this is an refresh or a first download
-        if self.last_id == None:
-            json = urllib2.urlopen('http://twitter.com/statuses/friends_timeline.json')
-        else:
-            #basically the twitter API will respond with just tweets newer than the ID we send
-            json = urllib2.urlopen('http://twitter.com/statuses/friends_timeline.json?since_id='+str(self.last_id)+'L')
-        #JSON is awesome stuff. we get given a long string of json encoded information
-        #which contains all the tweets, with lots of info, we decode to a json object
-        data = simplejson.loads(json.read())
-        #then this line does all the hard work. Basicaly for evey top level object in the JSON
-        #structure we call out getStatus method with the contents of the USER structure
-        #and the values of top level values text/id/created_at
-        [self.getStatus(x['user'],x['text'], x['id'], x['created_at'], "tweet") for x in data]
-                
+        try:
+    
+            if self.last_id == None:
+                json = urllib2.urlopen('http://twitter.com/statuses/friends_timeline.json')
+            else:
+                #basically the twitter API will respond with just tweets newer than the ID we send
+                json = urllib2.urlopen('http://twitter.com/statuses/friends_timeline.json?since_id='+str(self.last_id)+'L')
+            #JSON is awesome stuff. we get given a long string of json encoded information
+            #which contains all the tweets, with lots of info, we decode to a json object
+            data = simplejson.loads(json.read())
+            #then this line does all the hard work. Basicaly for evey top level object in the JSON
+            #structure we call out getStatus method with the contents of the USER structure
+            #and the values of top level values text/id/created_at
+            [self.getStatus(x['user'],x['text'], x['id'], x['created_at'], "tweet") for x in data]
+        except IOError, e:
+            
+            if hasattr(e, 'code'):
+                if (e.code == 401):
+                    reason = "Not authorised: check uid/pwd"
+                else:
+                    reason = ""
+                dialog = gtk.Dialog("Server returned "+ str(e.code) + " " +reason,
+                     self.window,
+                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+                dialog.show()
+
+          
         
     def getDMs(self, *args):
         print "getting DMs"
@@ -396,18 +413,29 @@ class Witter():
         opener = urllib2.build_opener(auth_handler)
         # ...and install it globally so it can be used with urlopen.
         urllib2.install_opener(opener)
-        #switch on whether this is an refresh or a first download
-        if self.last_dm_id == None:
-            json = urllib2.urlopen('http://twitter.com/direct_messages.json')
-        else:
-            json = urllib2.urlopen('http://twitter.com/direct_messages.json?since_id='+str(self.last_dm_id)+'L')
-        #JSON is awesome stuff. we get given a long string of json encoded information
-        #which contains all the tweets, with lots of info, we decode to a json object
-        data = simplejson.loads(json.read())
-        #then this line does all the hard work. Basicaly for evey top level object in the JSON
-        #structure we call out getStatus method with the contents of the USER structure
-        #and the values of top level values text/id/created_at
-        [self.getStatus(x['sender'],x['text'], x['id'], x['created_at'], "dm") for x in data]
+        try:
+            #switch on whether this is an refresh or a first download
+            if self.last_dm_id == None:
+                json = urllib2.urlopen('http://twitter.com/direct_messages.json')
+            else:
+                json = urllib2.urlopen('http://twitter.com/direct_messages.json?since_id='+str(self.last_dm_id)+'L')
+            #JSON is awesome stuff. we get given a long string of json encoded information
+            #which contains all the tweets, with lots of info, we decode to a json object
+            data = simplejson.loads(json.read())
+            #then this line does all the hard work. Basicaly for evey top level object in the JSON
+            #structure we call out getStatus method with the contents of the USER structure
+            #and the values of top level values text/id/created_at
+            [self.getStatus(x['sender'],x['text'], x['id'], x['created_at'], "dm") for x in data]
+        except IOError, e:
+            if hasattr(e, 'code'):
+                if (e.code == 401):
+                    reason = "Not authorised: check uid/pwd"
+                else:
+                    reason = ""
+                dialog = gtk.Dialog("Server returned "+ str(e.code) + " " +reason,
+                     self.window,
+                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+                dialog.show()
 
     def getMentions(self, *args):
         print "getting Mentions"
@@ -426,18 +454,29 @@ class Witter():
         opener = urllib2.build_opener(auth_handler)
         # ...and install it globally so it can be used with urlopen.
         urllib2.install_opener(opener)
-        #switch on whether this is an refresh or a first download
-        if self.last_mention_id == None:
-            json = urllib2.urlopen('http://twitter.com/statuses/mentions.json')
-        else:
-            json = urllib2.urlopen('http://twitter.com/statuses/mentions.json?since_id='+str(self.last_mention_id)+'L')
-        #JSON is awesome stuff. we get given a long string of json encoded information
-        #which contains all the tweets, with lots of info, we decode to a json object
-        data = simplejson.loads(json.read())
-        #then this line does all the hard work. Basicaly for evey top level object in the JSON
-        #structure we call out getStatus method with the contents of the USER structure
-        #and the values of top level values text/id/created_at
-        [self.getStatus(x['user'],x['text'], x['id'], x['created_at'], "mention") for x in data]
+        try:
+            #switch on whether this is an refresh or a first download
+            if self.last_mention_id == None:
+                json = urllib2.urlopen('http://twitter.com/statuses/mentions.json')
+            else:
+                json = urllib2.urlopen('http://twitter.com/statuses/mentions.json?since_id='+str(self.last_mention_id)+'L')
+            #JSON is awesome stuff. we get given a long string of json encoded information
+            #which contains all the tweets, with lots of info, we decode to a json object
+            data = simplejson.loads(json.read())
+            #then this line does all the hard work. Basicaly for evey top level object in the JSON
+            #structure we call out getStatus method with the contents of the USER structure
+            #and the values of top level values text/id/created_at
+            [self.getStatus(x['user'],x['text'], x['id'], x['created_at'], "mention") for x in data]
+        except IOError, e:
+            if hasattr(e, 'code'):
+                if (e.code == 401):
+                    reason = "Not authorised: check uid/pwd"
+                else:
+                    reason = ""
+                dialog = gtk.Dialog("Server returned "+ str(e.code) + " " +reason,
+                     self.window,
+                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+                dialog.show()
         
     def getPublic(self, *args):
         print "getting Public timeline"
@@ -455,19 +494,30 @@ class Witter():
         opener = urllib2.build_opener(auth_handler)
         # ...and install it globally so it can be used with urlopen.
         urllib2.install_opener(opener)
-        #switch on whether this is an refresh or a first download
-        if self.last_public_id == None:
-            json = urllib2.urlopen('http://twitter.com/statuses/public_timeline.json')
-        else:
-            json = urllib2.urlopen('http://twitter.com/statuses/public_timeline.json?since_id='+str(self.last_public_id)+'L')
-        #JSON is awesome stuff. we get given a long string of json encoded information
-        #which contains all the tweets, with lots of info, we decode to a json object
-        data = simplejson.loads(json.read())
-        #then this line does all the hard work. Basicaly for evey top level object in the JSON
-        #structure we call out getStatus method with the contents of the USER structure
-        #and the values of top level values text/id/created_at
-        [self.getStatus(x['user'],x['text'], x['id'], x['created_at'], "public") for x in data]
-        
+        try:
+            #switch on whether this is an refresh or a first download
+            if self.last_public_id == None:
+                json = urllib2.urlopen('http://twitter.com/statuses/public_timeline.json')
+            else:
+                json = urllib2.urlopen('http://twitter.com/statuses/public_timeline.json?since_id='+str(self.last_public_id)+'L')
+            #JSON is awesome stuff. we get given a long string of json encoded information
+            #which contains all the tweets, with lots of info, we decode to a json object
+            data = simplejson.loads(json.read())
+            #then this line does all the hard work. Basicaly for evey top level object in the JSON
+            #structure we call out getStatus method with the contents of the USER structure
+            #and the values of top level values text/id/created_at
+            [self.getStatus(x['user'],x['text'], x['id'], x['created_at'], "public") for x in data]
+        except IOError, e:
+            if hasattr(e, 'code'):
+                if (e.code == 401):
+                    reason = "Not authorised: check uid/pwd"
+                else:
+                    reason = ""
+                dialog = gtk.Dialog("Server returned "+ str(e.code) + " " +reason,
+                     self.window,
+                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+                dialog.show()
+            
     def getSearch(self, *args):
         print "performing search"
         
@@ -501,19 +551,29 @@ class Witter():
         # ...and install it globally so it can be used with urlopen.
         urllib2.install_opener(opener)
         
-        
-        json = urllib2.urlopen('http://search.twitter.com/search.json?'+search)
-        
-        #JSON is awesome stuff. we get given a long string of json encoded information
-        #which contains all the tweets, with lots of info, we decode to a json object
-        data = simplejson.loads(json.read())
-        #then this line does all the hard work. Basicaly for evey top level object in the JSON
-        #structure we call out getStatus method with the contents of the USER structure
-        #and the values of top level values text/id/created_at
-        print data
-        results = data['results']
-        [self.getStatus(x['from_user'],x['text'], x['id'], x['created_at'], "search") for x in results]
-        
+        try:
+            json = urllib2.urlopen('http://search.twitter.com/search.json?'+search)
+            
+            #JSON is awesome stuff. we get given a long string of json encoded information
+            #which contains all the tweets, with lots of info, we decode to a json object
+            data = simplejson.loads(json.read())
+            #then this line does all the hard work. Basicaly for evey top level object in the JSON
+            #structure we call out getStatus method with the contents of the USER structure
+            #and the values of top level values text/id/created_at
+            print data
+            results = data['results']
+            [self.getStatus(x['from_user'],x['text'], x['id'], x['created_at'], "search") for x in results]
+        except IOError, e:
+            if hasattr(e, 'code'):
+                if (e.code == 401):
+                    reason = "Not authorised: check uid/pwd"
+                else:
+                    reason = ""
+                dialog = gtk.Dialog("Server returned "+ str(e.code) + " " +reason,
+                     self.window,
+                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+                dialog.show()
+            
     def getTrends(self, *args):
         print "getting Trending topics"
         #first clear the previous 10
@@ -533,18 +593,28 @@ class Witter():
         # ...and install it globally so it can be used with urlopen.
         urllib2.install_opener(opener)
         
-        
-        json = urllib2.urlopen('http://search.twitter.com/trends.json')
-        #JSON is awesome stuff. we get given a long string of json encoded information
-        #which contains all the tweets, with lots of info, we decode to a json object
-        data = simplejson.loads(json.read())
-        print data
-        #then this line does all the hard work. Basicaly for evey top level object in the JSON
-        #structure we call out getStatus method with the contents of the USER structure
-        #and the values of top level values text/id/created_at
-        trends = data['trends']
-        [self.getTrend(x['name'], x['url']) for x in trends]
-        
+        try:
+            json = urllib2.urlopen('http://search.twitter.com/trends.json')
+            #JSON is awesome stuff. we get given a long string of json encoded information
+            #which contains all the tweets, with lots of info, we decode to a json object
+            data = simplejson.loads(json.read())
+            print data
+            #then this line does all the hard work. Basicaly for evey top level object in the JSON
+            #structure we call out getStatus method with the contents of the USER structure
+            #and the values of top level values text/id/created_at
+            trends = data['trends']
+            [self.getTrend(x['name'], x['url']) for x in trends]
+        except IOError, e:
+            if hasattr(e, 'code'):
+                if (e.code == 401):
+                    reason = "Not authorised: check uid/pwd"
+                else:
+                    reason = ""
+                dialog = gtk.Dialog("Server returned "+ str(e.code) + " " +reason,
+                     self.window,
+                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+                dialog.show()
+            
     def getTrend(self, name, url):
         
         self.trendliststore.append([name, self.namecolour,url,self.tweetcolour])
@@ -569,23 +639,32 @@ class Witter():
         # ...and install it globally so it can be used with urlopen.
         urllib2.install_opener(opener)
         
-        
-        json = urllib2.urlopen('http://twitter.com/statuses/friends.json')
-        #JSON is awesome stuff. we get given a long string of json encoded information
-        #which contains all the tweets, with lots of info, we decode to a json object
-        data = simplejson.loads(json.read())
-        print data
-        #then this line does all the hard work. Basicaly for evey top level object in the JSON
-        #structure we call out getStatus method with the contents of the USER structure
-        #and the values of top level values text/id/created_at
-        
-        for x in data:
-            #if we follow someone with no status then you get a key error on status
-            try:
-                self.getStatus(x['screen_name'],x['status'], x['id'], x['created_at'], "friend") 
-            except KeyError:
-                print  x
-        
+        try:
+            json = urllib2.urlopen('http://twitter.com/statuses/friends.json')
+            #JSON is awesome stuff. we get given a long string of json encoded information
+            #which contains all the tweets, with lots of info, we decode to a json object
+            data = simplejson.loads(json.read())
+            
+            #then this line does all the hard work. Basicaly for evey top level object in the JSON
+            #structure we call out getStatus method with the contents of the USER structure
+            #and the values of top level values text/id/created_at
+            
+            for x in data:
+                #if we follow someone with no status then you get a key error on status
+                try:
+                    self.getStatus(x['screen_name'],x['status'], x['id'], x['created_at'], "friend") 
+                except KeyError:
+                    print  x
+        except IOError, e:
+            if hasattr(e, 'code'):
+                if (e.code == 401):
+                    reason = "Not authorised: check uid/pwd"
+                else:
+                    reason = ""
+                dialog = gtk.Dialog("Server returned "+ str(e.code) + " " +reason,
+                     self.window,
+                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+                dialog.show()    
         
     def getStatus(self, user,data, id, created_at, type):
         #at this point user is another JSON structure of lots more values of which we are currently
@@ -595,7 +674,8 @@ class Witter():
         # and black for the tweet. At some point I intend to do some alternating colours for
         # cell backgrounds to make the display clearer
         if (re.search("tweet", type)):
-            self.liststore.append([ "@"+user['screen_name'],self.namecolour,data+"\nposted on: "+created_at,self.tweetcolour, id, type])
+            data = data.replace("&","&amp;")
+            self.liststore.append([ "@"+user['screen_name'],self.namecolour,"<span foreground=\"#0000FF\"><b>@"+user['screen_name']+"</b></span> : "+data+"\n<span size=\"xx-small\">posted on: "+created_at+"</span>",self.tweetcolour, id, type])
             #now we process the id, this is so we can do a refresh with just the posts since the latest one we have
             #if we haven't stored the most recent id then store this one
             if self.last_id == None:
@@ -605,7 +685,7 @@ class Witter():
                 if long(self.last_id) < long(id):
                     self.last_id=id
         elif (re.search("dm", type)):
-            self.dmliststore.append([ "@"+user['screen_name'],self.namecolour,data+"\nposted on: "+created_at,self.tweetcolour, id, type])
+            self.dmliststore.append([ "@"+user['screen_name'],self.namecolour,"<span foreground=\"#0000FF\"><b>@"+user['screen_name']+"</b></span> : "+data+"\n<span size=\"xx-small\">posted on: "+created_at+"</span>",self.tweetcolour, id, type])
             if self.last_dm_id == None:
                 self.last_dm_id=id
             else:
@@ -613,7 +693,7 @@ class Witter():
                 if long(self.last_dm_id) < long(id):
                     self.last_dm_id=id
         elif (re.search("mention", type)):
-            self.mentionliststore.append([ "@"+user['screen_name'],self.namecolour,data+"\nposted on: "+created_at,self.tweetcolour, id, type])
+            self.mentionliststore.append([ "@"+user['screen_name'],self.namecolour,"<span foreground=\"#0000FF\"><b>@"+user['screen_name']+"</b></span> : "+data+"\n<span size=\"xx-small\">posted on: "+created_at+"</span>",self.tweetcolour, id, type])
             if self.last_mention_id == None:
                 self.last_mention_id=id
             else:
@@ -621,7 +701,7 @@ class Witter():
                 if long(self.last_mention_id) < long(id):
                     self.last_mention_id=id
         elif (re.search("public", type)):
-            self.publicliststore.append([ "@"+user['screen_name'],self.namecolour,data+"\nposted on: "+created_at,self.tweetcolour, id, type])
+            self.publicliststore.append([ "@"+user['screen_name'],self.namecolour,"<span foreground=\"#0000FF\"><b>@"+user['screen_name']+"</b></span> : "+data+"\n<span size=\"xx-small\">posted on: "+created_at+"</span>",self.tweetcolour, id, type])
             if self.last_public_id == None:
                 self.last_public_id=id
             else:
@@ -629,9 +709,9 @@ class Witter():
                 if long(self.last_public_id) < long(id):
                     self.last_public_id=id
         elif (re.search("friend", type)):
-            self.friendsliststore.append([ "@"+user,self.namecolour,data['text']+"\nposted on: "+data['created_at'],self.tweetcolour, id, type])
+            self.friendsliststore.append([ "@"+user,self.namecolour,"<span foreground=\"#0000FF\"><b>@"+user+"</b></span> : "+data['text']+"\n<span size=\"xx-small\">posted on: "+data['created_at']+"</span>",self.tweetcolour, id, type])
         elif (re.search("search", type)):
-            self.searchliststore.append([ "@"+user,self.namecolour,data+"\nposted on: "+created_at,self.tweetcolour, id, type])
+            self.searchliststore.append([ "@"+user,self.namecolour,"<span foreground=\"#0000FF\"><b>@"+user['screen_name']+"</b></span> : "+data+"\n<span size=\"xx-small\">posted on: "+created_at+"</span>",self.tweetcolour, id, type])
             
                 
     def newTweet(self, widget, *args):
@@ -840,13 +920,11 @@ class Witter():
                 print "found n900"
                 self.tweetcolour= "#FFFFFF"
                 self.namecolour="#FE00B8"
-                self.textcolour="#FFFFFF"
-                self.defaultwidth=560
+                self.defaultwidth=790
                 self.maemo_ver=5
             else:
                 print "found"+ read_data
                 self.maemo_ver=4
-                self.textcolour="#000000"
         except IOError:
             #couldn't find the file 
             print "Assuming pre-maemo5"
